import uuid, re, sys, muid, requests, time
from muid.mining import mine_once
from getjson import getjson

def testing_url():
    return "https://devapi.microprediction.org"

def default_url():
    return "https://api.microprediction.org"

class KeyConventions():

    @staticmethod
    def is_valid_key(key):
        """ Check if key is hash-memorable """
        return isinstance(key,str) and muid.validate(key)

    @staticmethod
    def create_key(difficulty=6):
        """ Create new write_key (string, not bytes) """
        return muid.create(difficulty=difficulty).decode_meme_stock()

    @staticmethod
    def maybe_create_key(seconds=1,difficulty=12):
        """ Find a MUID, maybe
             difficulty:  int  minimum length of the memorable part of the hash
        """
        quota = 100000000
        count = 0
        start_time = time.time()
        dffclty = difficulty
        while time.time()-start_time<seconds:
            report, dffclty, count = mine_once(dffclty, count, quota)
            if report:
                return report[0]["key"]


new_key = KeyConventions.create_key

class NameConventions(object):

    @staticmethod
    def sep():
        return '::'

    @staticmethod
    def is_plain_name(name: str):
        return NameConventions.is_valid_name(name) and not "~" in name

    @staticmethod
    def is_valid_name(name: str):
        name_regex = re.compile(r'^[-a-zA-Z0-9_~.:]{1,200}\.[json,html]+$', re.IGNORECASE)
        return (re.match(name_regex, name) is not None) and (not NameConventions.sep() in name)

    @staticmethod
    def random_name():
        return str(uuid.uuid4()) + '.json'

    def horizon_name(self, name, delay):
        """ Convention is used for performance and other hashes """  # Move to MicroConventions
        return str(delay)+self.sep() + name

    def split_horizon_name(self, key):
        spl = key.split(self.sep())
        name = spl[1]
        delay = int(spl[0])
        return name, delay

class ValueConventions(object):

    @staticmethod
    def is_scalar_value(value):
        try:
            fv = float(value)
            return True
        except:
            return False

    @staticmethod
    def is_valid_value(value):
        return isinstance(value, (str, int, float)) and sys.getsizeof(value) < 100000

    @staticmethod
    def is_small_value(value):
        """ Used to determine how to store history for dict like values """
        return sys.getsizeof(value) < 1200


class StatsConventions():

    @staticmethod
    def percentile_abscissa():
        return [-8., -5., -4, -3, -2.5, -2.0, -1.5, -1.25, -1, -0.8, -0.6, -0.4, -0.2, -0.1, -0.05, -0.02, -0.01, 0.,
                0.01, 0.02, 0.05, 0.1, 0.2, 0.4, 0.6, 0.8, 1., 1.25, 1.5, 2.0, 2.5, 3., 4., 5., 8.]


class MicroConventions(NameConventions, ValueConventions, StatsConventions, KeyConventions):

    def __init__(self, base_url=None, num_predictions=None, min_len=None, min_balance=None, delays=None) :
        """ Establish connection and adopt configuration parameters from site, if not provided """
        self.base_url       = base_url or default_url()
        default_config      = getjson(url='https://config.microprediction.org/config.json',
                             failover_url='https://stableconfig.microprediction.org/config.json')
        self.delays          = delays or default_config["delays"]
        self.num_predictions = num_predictions or default_config["num_predictions"]
        self.min_len         = min_len or default_config["min_len"]
        self.min_balance     = min_balance or default_config["min_balance"]






from microconventions import MicroConventions, KeyConventions, api_url, failover_api_url, new_key, create_key, maybe_create_key

PY_REDIS_ARGS = (
    'host', 'port', 'db', 'username', 'password', 'socket_timeout', 'socket_keepalive', 'socket_keepalive_options',
    'connection_pool', 'unix_socket_path', 'encoding', 'encoding_errors', 'charset', 'errors',
    'decode_responses', 'retry_on_timeout', 'ssl', 'ssl_keyfile', 'ssl_certfile', 'ssl_cert_reqs', 'ssl_ca_certs',
    'ssl_check_hostname', 'max_connections', 'single_connection_client', 'health_check_interval', 'client_name')
FAKE_REDIS_ARGS = ('decode_responses',)


from re import findall
from functools import reduce
import math
# Utilities for the Meme stock of the hour streams


def decode_meme_stock_old(encoded):
    """Decode an integer to a ticker."""
    return str(reduce(lambda x,y:x+y,map(
        lambda x:chr(int(x)),findall(r"[0-9][0-9]",
        str(int(encoded))))))


def encode_meme_stock_old(decoded):
    """Encode a ticker as an integer."""
    return int(reduce(lambda x,y:x+y,map(
         lambda x:str(ord(x)),list(decoded))))


def decode_meme_stock(encoded):
    return decode_meme_stock_old(100*encoded)


def encode_meme_stock(decoded):
    encoded = encode_meme_stock_old(decoded)
    return encoded/100.0